@@ -1,19 +1,22 @@
 import atexit
 import os
 import sys
+from contextlib import asynccontextmanager
 from enum import Enum
 from itertools import count
-from typing import Type, Callable, Any
+from typing import Type, Callable, Any, TypeVar
 
 import attr
-from async_generator import asynccontextmanager
+import trio
 
 from ._proc import WORKER_PROC_MAP
 from ._abc import WorkerCache, AbstractWorker, NoPublicConstructor
 
+T = TypeVar("T")
+
 # Sane default might be to expect cpu-bound work
 DEFAULT_LIMIT = os.cpu_count() or 1
-limiter_runvar = None
+limiter_runvar = trio.lowlevel.RunVar("trio_parallel")
 ATEXIT_SHUTDOWN_GRACE_PERIOD = 30.0
 
 
@@ -26,11 +29,6 @@ def current_default_worker_limiter():
     is initialized to the number of CPUs reported by :func:`os.cpu_count`.
 
     """
-    import trio
-
-    global limiter_runvar
-    if limiter_runvar is None:
-        limiter_runvar = trio.lowlevel.RunVar("trio_parallel")
     try:
         return limiter_runvar.get()
     except LookupError:
@@ -60,7 +58,6 @@ recommended."""
 
 @attr.s(slots=True, eq=False)
 class ContextLifetimeManager:
-    closed = attr.ib(False)
     task = attr.ib(None)
     # Counters are used for thread safety of the default cache
     enter_counter = attr.ib(factory=lambda: count(1))
@@ -68,32 +65,26 @@ class ContextLifetimeManager:
 
     async def __aenter__(self):
         # only async to save indentation
-        if self.closed:
-            import trio
-
+        if self.task:
             raise trio.ClosedResourceError
         next(self.enter_counter)
 
     async def __aexit__(self, exc_type, exc_val, exc_tb):
         # only async to save indentation
         next(self.exit_counter)
-        if self.task and self.calc_running() == 0:
-            import trio
-
-            trio.lowlevel.reschedule(self.task)
+        if self.task:
+            if self.calc_running() == 0:
+                trio.lowlevel.reschedule(self.task)
 
     async def close_and_wait(self):
-        assert not self.closed
-        self.closed = True
+        assert not self.task
+        self.task = trio.lowlevel.current_task()
         if self.calc_running() != 0:
-            import trio
 
-            self.task = trio.lowlevel.current_task()
-            await trio.lowlevel.wait_task_rescheduled(
-                # never cancelled anyway
-                lambda _: trio.lowlevel.Abort.FAILED  # pragma: no cover
-            )
-            self.task = None
+            def abort_func(raise_cancel):  # pragma: no cover
+                return trio.lowlevel.Abort.FAILED  # never cancelled anyway
+
+            await trio.lowlevel.wait_task_rescheduled(abort_func)
 
     def calc_running(self):
         # __reduce__ is the only count API that can extract the internal int value
@@ -161,16 +152,21 @@ class WorkerContext(metaclass=NoPublicConstructor):
         self.__dict__["_worker_class"] = worker_class
         self.__dict__["_worker_cache"] = cache_class()
 
-    async def run_sync(self, sync_fn, *args, cancellable=False, limiter=None):
-        """Run ``sync_fn(*args)`` in a separate process and return/raise it's outcome.
+    @trio.lowlevel.enable_ki_protection
+    async def run_sync(
+        self,
+        sync_fn: Callable[..., T],
+        *args,
+        cancellable: bool = False,
+        limiter: trio.CapacityLimiter = None,
+    ) -> T:
+        """Run ``sync_fn(*args)`` in a separate process and return/raise its outcome.
 
         Behaves according to the customized attributes of the context. See
         :func:`trio_parallel.run_sync()` for details.
 
         Raises:
             trio.ClosedResourceError: if this method is run on a closed context"""
-        import trio
-
         if limiter is None:
             limiter = current_default_worker_limiter()
 
@@ -196,14 +192,13 @@ class WorkerContext(metaclass=NoPublicConstructor):
                     return result.unwrap()
 
     async def _aclose(self, grace_period=None):
-        import trio
-
         if grace_period is None:
             grace_period = self.grace_period
         with trio.CancelScope(shield=True):
             await self._lifetime.close_and_wait()
             await trio.to_thread.run_sync(self._worker_cache.shutdown, grace_period)
 
+    @trio.lowlevel.enable_ki_protection
     def statistics(self):
         self._worker_cache.prune()
         return WorkerContextStatistics(
@@ -214,33 +209,31 @@ class WorkerContext(metaclass=NoPublicConstructor):
 
 # intentionally skip open_worker_context
 DEFAULT_CONTEXT = WorkerContext._create()
-DEFAULT_CONTEXT_RUNVAR = None
+DEFAULT_CONTEXT_RUNVAR = trio.lowlevel.RunVar("win32_ctx")
 if sys.platform == "win32":
 
-    async def close_at_run_end(ctx):
-        import trio
+    # TODO: intelligently test ki protection here such that CI fails if the
+    #  decorators disappear
 
+    @trio.lowlevel.enable_ki_protection
+    async def close_at_run_end(ctx):
         try:
             await trio.sleep_forever()
         finally:
+            # KeyboardInterrupt here could leak the context
             await ctx._aclose(ATEXIT_SHUTDOWN_GRACE_PERIOD)
 
+    @trio.lowlevel.enable_ki_protection
     def get_default_context():
-        import trio
-
-        global DEFAULT_CONTEXT_RUNVAR
-
-        if DEFAULT_CONTEXT_RUNVAR is None:
-            DEFAULT_CONTEXT_RUNVAR = trio.lowlevel.RunVar("win32_ctx")
         try:
             ctx = DEFAULT_CONTEXT_RUNVAR.get()
         except LookupError:
             ctx = WorkerContext._create()
             DEFAULT_CONTEXT_RUNVAR.set(ctx)
+            # KeyboardInterrupt here could leak the context
             trio.lowlevel.spawn_system_task(close_at_run_end, ctx)
         return ctx
 
-
 else:
 
     def get_default_context():
@@ -268,6 +261,7 @@ def default_context_statistics():
 
 
 @asynccontextmanager
+@trio.lowlevel.enable_ki_protection
 async def open_worker_context(
     idle_timeout=DEFAULT_CONTEXT.idle_timeout,
     init=DEFAULT_CONTEXT.init,
@@ -351,8 +345,13 @@ def atexit_shutdown_grace_period(grace_period=-1.0):
     return ATEXIT_SHUTDOWN_GRACE_PERIOD
 
 
-async def run_sync(sync_fn, *args, cancellable=False, limiter=None):
-    """Run ``sync_fn(*args)`` in a separate process and return/raise it's outcome.
+async def run_sync(
+    sync_fn: Callable[..., T],
+    *args,
+    cancellable: bool = False,
+    limiter: trio.CapacityLimiter = None,
+) -> T:
+    """Run ``sync_fn(*args)`` in a separate process and return/raise its outcome.
 
     This function is intended to enable the following:
 
