@@ -8,7 +8,7 @@ import math
 import trio
 import pytest
 
-from ._funcs import (
+from _trio_parallel_workers._funcs import (
     _lambda,
     _return_lambda,
     _raise_ki,
@@ -23,6 +23,7 @@ from .._abc import BrokenWorkerError
 @pytest.fixture(params=list(WORKER_PROC_MAP.values()), ids=list(WORKER_PROC_MAP.keys()))
 async def worker(request):
     worker = request.param[0](math.inf, bool, bool)
+    await worker.start()
     try:
         yield worker
     finally:
@@ -40,7 +41,6 @@ async def worker(request):
 
 
 async def test_run_sync_cancel_infinite_loop(worker, manager):
-    await worker.start()
     ev = manager.Event()
 
     async with trio.open_nursery() as nursery:
@@ -48,11 +48,10 @@ async def test_run_sync_cancel_infinite_loop(worker, manager):
         await trio.to_thread.run_sync(ev.wait, cancellable=True)
         nursery.cancel_scope.cancel()
     with trio.fail_after(1):
-        assert await worker.wait() in (-15, -9, 255)  # 255 for py3.6 forkserver
+        assert await worker.wait() in (-15, -9)
 
 
 async def test_run_sync_raises_on_kill(worker, manager):
-    await worker.start()
     ev = manager.Event()
     await worker.run_sync(int)  # running start so actual test is less racy
     with pytest.raises(BrokenWorkerError) as exc_info, trio.fail_after(5):
@@ -61,12 +60,11 @@ async def test_run_sync_raises_on_kill(worker, manager):
             await trio.to_thread.run_sync(ev.wait, cancellable=True)
             worker.kill()  # also tests multiple calls to worker.kill
     exitcode = await worker.wait()
-    assert exitcode in (-15, -9, 255)  # 255 for py3.6 forkserver
+    assert exitcode in (-15, -9)
     assert exc_info.value.args[-1].exitcode == exitcode
 
 
 async def test_run_sync_raises_on_segfault(worker, capfd):
-    await worker.start()
     # This test was flaky on CI across several platforms and implementations.
     # I can reproduce it locally if there is some other process using the rest
     # of the CPU (F@H in this case) although I cannot explain why running this
@@ -95,26 +93,23 @@ async def test_run_sync_raises_on_segfault(worker, capfd):
 
 
 async def test_exhaustively_cancel_run_sync(worker, manager):
-    await worker.start()
     # cancel at job send if we reuse the worker
     ev = manager.Event()
     await worker.run_sync(int)
     with trio.fail_after(1):
         with trio.move_on_after(0):
             await worker.run_sync(_never_halts, ev)
-        assert await worker.wait() in (-15, -9, 255)  # 255 for py3.6 forkserver
+        assert await worker.wait() in (-15, -9)
 
     # cancel at result recv is tested elsewhere
 
 
 async def test_ki_does_not_propagate(worker):
-    await worker.start()
     (await worker.run_sync(_raise_ki)).unwrap()
 
 
 @pytest.mark.parametrize("job", [_lambda, _return_lambda])
 async def test_unpickleable(job, worker):
-    await worker.start()
     from pickle import PicklingError
 
     with pytest.raises((PicklingError, AttributeError)):
@@ -124,5 +119,4 @@ async def test_unpickleable(job, worker):
 async def test_no_trio_in_subproc(worker):
     if worker.mp_context._name == "fork":
         pytest.skip("Doesn't matter on ForkProcWorker")
-    await worker.start()
     assert (await worker.run_sync(_no_trio)).unwrap()
